@@ -32,6 +32,7 @@ static const StringMappingType TOKEN_MAP[] = {
    { "ClockStyle",         TOK_CLOCKSTYLE       },
    { "Close",              TOK_CLOSE            },
    { "Corner",             TOK_CORNER           },
+   { "DefaultIcon",        TOK_DEFAULTICON      },
    { "Desktop",            TOK_DESKTOP          },
    { "Desktops",           TOK_DESKTOPS         },
    { "Dock",               TOK_DOCK             },
@@ -80,9 +81,11 @@ static const StringMappingType TOKEN_MAP[] = {
    { "Stick",              TOK_STICK            },
    { "Swallow",            TOK_SWALLOW          },
    { "TaskList",           TOK_TASKLIST         },
+   { "TaskListStyle",      TOK_TASKLISTSTYLE    },
    { "Text",               TOK_TEXT             },
    { "Tray",               TOK_TRAY             },
    { "TrayButton",         TOK_TRAYBUTTON       },
+   { "TrayButtonStyle",    TOK_TRAYBUTTONSTYLE  },
    { "TrayStyle",          TOK_TRAYSTYLE        },
    { "Width",              TOK_WIDTH            },
    { "WindowStyle",        TOK_WINDOWSTYLE      }
