@@ -37,6 +37,7 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <fcntl.h>
+#include <errno.h>
 
 /** Mapping of key names to key types.
  * Note that this mapping must be sorted.
@@ -88,6 +89,7 @@ static const unsigned int KEY_MAP_COUNT = ARRAY_LENGTH(KEY_MAP);
  * Note that this mapping must be sorted.
  */
 static const StringMappingType OPTION_MAP[] = {
+   { "aerosnap",           OPTION_AEROSNAP      },
    { "border",             OPTION_BORDER        },
    { "centered",           OPTION_CENTERED      },
    { "constrain",          OPTION_CONSTRAIN     },
@@ -102,6 +104,7 @@ static const StringMappingType OPTION_MAP[] = {
    { "minimized",          OPTION_MINIMIZED     },
    { "noborder",           OPTION_NOBORDER      },
    { "noclose",            OPTION_NOCLOSE       },
+   { "nodrag",             OPTION_NODRAG        },
    { "nofocus",            OPTION_NOFOCUS       },
    { "nofullscreen",       OPTION_NOFULLSCREEN  },
    { "nolist",             OPTION_NOLIST        },
@@ -182,13 +185,14 @@ static void ParseGroupOption(const TokenNode *tp,
 /* Style. */
 static void ParseWindowStyle(const TokenNode *tp);
 static void ParseActiveWindowStyle(const TokenNode *tp);
-static void ParseTrayStyle(const TokenNode *tp);
-static void ParseActiveTrayStyle(const TokenNode *tp);
+static void ParseTrayStyle(const TokenNode *tp,
+                           FontType font, ColorType baseColor);
+static void ParseActive(const TokenNode *tp, ColorType fg,
+                        ColorType bg1, ColorType bg2,
+                        ColorType up, ColorType down);
 static void ParsePagerStyle(const TokenNode *tp);
-static void ParseActivePagerStyle(const TokenNode *tp);
 static void ParseClockStyle(const TokenNode *tp);
 static void ParseMenuStyle(const TokenNode *tp);
-static void ParseActiveMenuStyle(const TokenNode *tp);
 static void ParsePopupStyle(const TokenNode *tp);
 
 /* Feel. */
@@ -199,7 +203,7 @@ static void ParseResizeMode(const TokenNode *tp);
 static void ParseFocusModel(const TokenNode *tp);
 
 static AlignmentType ParseTextAlignment(const TokenNode *tp);
-static DecorationsType ParseDecorations(const TokenNode *tp);
+static void ParseDecorations(const TokenNode *tp, DecorationsType *deco);
 static void ParseGradient(const char *value, ColorType a, ColorType b);
 static char *FindAttribute(AttributeNode *ap, const char *name);
 static int ParseTokenValue(const StringMappingType *mapping, int count,
@@ -336,7 +340,13 @@ void Parse(const TokenNode *start, int depth)
                ParseTray(tp);
                break;
             case TOK_TRAYSTYLE:
-               ParseTrayStyle(tp);
+               ParseTrayStyle(tp, FONT_TRAY, COLOR_TRAY_FG);
+               break;
+            case TOK_TASKLISTSTYLE:
+               ParseTrayStyle(tp, FONT_TASKLIST, COLOR_TASKLIST_FG);
+               break;
+            case TOK_TRAYBUTTONSTYLE:
+               ParseTrayStyle(tp, FONT_TRAYBUTTON, COLOR_TRAYBUTTON_FG);
                break;
             case TOK_CLOCKSTYLE:
                ParseClockStyle(tp);
@@ -359,6 +369,9 @@ void Parse(const TokenNode *start, int depth)
             case TOK_BUTTONMENU:
                SetBorderIcon(BI_MENU, tp->value);
                break;
+            case TOK_DEFAULTICON:
+               SetDefaultIcon(tp->value);
+               break;
             default:
                InvalidTag(tp, TOK_JWM);
                break;
@@ -855,7 +868,7 @@ void ParseWindowStyle(const TokenNode *tp)
 {
    const TokenNode *np;
 
-   settings.windowDecorations = ParseDecorations(tp);
+   ParseDecorations(tp, &settings.windowDecorations);
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_FONT:
@@ -1022,44 +1035,59 @@ void ParseDesktopBackground(int desktop, const TokenNode *tp)
 }
 
 /** Parse tray style. */
-void ParseTrayStyle(const TokenNode *tp)
+void ParseTrayStyle(const TokenNode *tp, FontType font, ColorType fg)
 {
    const TokenNode *np;
-   const char *temp;
+   const ColorType bg1 = fg + COLOR_TRAY_BG1 - COLOR_TRAY_FG;
+   const ColorType bg2 = fg + COLOR_TRAY_BG2 - COLOR_TRAY_FG;
+   const ColorType activeFg = fg + COLOR_TRAY_ACTIVE_FG - COLOR_TRAY_FG;
+   const ColorType activeBg1 = fg + COLOR_TRAY_ACTIVE_BG1 - COLOR_TRAY_FG;
+   const ColorType activeBg2 = fg + COLOR_TRAY_ACTIVE_BG2 - COLOR_TRAY_FG;
+   const ColorType up = fg + COLOR_TRAY_UP - COLOR_TRAY_FG;
+   const ColorType down = fg + COLOR_TRAY_DOWN - COLOR_TRAY_FG;
+   const ColorType activeUp = fg + COLOR_TRAY_ACTIVE_UP - COLOR_TRAY_FG;
+   const ColorType activeDown = fg + COLOR_TRAY_ACTIVE_DOWN - COLOR_TRAY_FG;
 
-   settings.trayDecorations = ParseDecorations(tp);
-   temp = FindAttribute(tp->attributes, "group");
-   if(temp) {
-      settings.groupTasks = !strcmp(temp, TRUE_VALUE);
-   }
+   /* TrayStyle has extra attributes. */
+   if(tp->type == TOK_TRAYSTYLE) {
+      const char *temp;
+      ParseDecorations(tp, &settings.trayDecorations);
+      temp = FindAttribute(tp->attributes, "group");
+      if(temp) {
+         settings.groupTasks = !strcmp(temp, TRUE_VALUE);
+      }
 
-   temp = FindAttribute(tp->attributes, "list");
-   if(temp) {
-      settings.listAllTasks = !strcmp(temp, "all");
+      temp = FindAttribute(tp->attributes, "list");
+      if(temp) {
+         settings.listAllTasks = !strcmp(temp, "all");
+      }
    }
 
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_FONT:
-         SetFont(FONT_TRAY, np->value);
+         SetFont(font, np->value);
          break;
       case TOK_ACTIVE:
-         ParseActiveTrayStyle(np);
+         ParseActive(np, activeFg, activeBg1, activeBg2, activeUp, activeDown);
          break;
       case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_TRAY_BG1, COLOR_TRAY_BG2);
+         ParseGradient(np->value, bg1, bg2);
          break;
       case TOK_FOREGROUND:
-         SetColor(COLOR_TRAY_FG, np->value);
+         SetColor(fg, np->value);
          break;
       case TOK_OUTLINE:
-         ParseGradient(np->value, COLOR_TRAY_DOWN, COLOR_TRAY_UP);
+         ParseGradient(np->value, down, up);
          break;
       case TOK_OPACITY:
-         settings.trayOpacity = ParseOpacity(np, np->value);
-         break;
+         if(tp->type == TOK_TRAYSTYLE) {
+            settings.trayOpacity = ParseOpacity(np, np->value);
+            break;
+         }
+         /* fall through */
       default:
-         InvalidTag(np, TOK_TRAYSTYLE);
+         InvalidTag(np, tp->type);
          break;
       }
    }
@@ -1067,18 +1095,30 @@ void ParseTrayStyle(const TokenNode *tp)
 }
 
 /** Parse active tray style. */
-void ParseActiveTrayStyle(const TokenNode *tp)
+void ParseActive(const TokenNode *tp, ColorType fg,
+                 ColorType bg1, ColorType bg2,
+                 ColorType up, ColorType down)
 {
    const TokenNode *np;
 
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_TRAY_ACTIVE_BG1, COLOR_TRAY_ACTIVE_BG2);
+         if(bg1 == bg2) {
+            SetColor(bg1, np->value);
+         } else {
+            ParseGradient(np->value, bg1, bg2);
+         }
          break;
       case TOK_FOREGROUND:
-         SetColor(COLOR_TRAY_ACTIVE_FG, np->value);
+         SetColor(fg, np->value);
          break;
+      case TOK_OUTLINE:
+         if(up != COLOR_COUNT) {
+            ParseGradient(np->value, down, up);
+            break;
+         }
+         /* fall through */
       default:
          InvalidTag(np, TOK_ACTIVE);
          break;
@@ -1216,6 +1256,71 @@ void ParseTaskList(const TokenNode *tp, TrayType *tray)
    if(temp) {
       SetTaskBarHeight(cp, temp);
    }
+
+   temp = FindAttribute(tp->attributes, "labeled");
+   if(temp && !strcmp(temp, FALSE_VALUE)) {
+      SetTaskBarLabeled(cp, 0);
+   }
+}
+
+/** Parse the task list style. */
+void ParseTaskListStyle(const TokenNode *tp)
+{
+   const TokenNode *np;
+   for(np = tp->subnodeHead; np; np = np->next) {
+      switch(np->type) {
+      case TOK_FONT:
+         SetFont(FONT_TASKLIST, np->value);
+         break;
+      case TOK_ACTIVE:
+         ParseActive(np, COLOR_TASKLIST_ACTIVE_FG,
+                     COLOR_TASKLIST_ACTIVE_BG1, COLOR_TASKLIST_ACTIVE_BG2,
+                     COLOR_TASKLIST_ACTIVE_UP, COLOR_TASKLIST_ACTIVE_DOWN);
+         break;
+      case TOK_BACKGROUND:
+         ParseGradient(np->value, COLOR_TASKLIST_BG1, COLOR_TASKLIST_BG2);
+         break;
+      case TOK_FOREGROUND:
+         SetColor(COLOR_TASKLIST_FG, np->value);
+         break;
+      case TOK_OUTLINE:
+         ParseGradient(np->value, COLOR_TASKLIST_DOWN, COLOR_TASKLIST_UP);
+         break;
+      default:
+         InvalidTag(np, TOK_TASKLISTSTYLE);
+         break;
+      }
+   }
+}
+
+/** Parse the tray button style. */
+void ParseTrayButtonStyle(const TokenNode *tp)
+{
+   const TokenNode *np;
+   for(np = tp->subnodeHead; np; np = np->next) {
+      switch(np->type) {
+      case TOK_FONT:
+         SetFont(FONT_TRAYBUTTON, np->value);
+         break;
+      case TOK_ACTIVE:
+         ParseActive(np, COLOR_TRAYBUTTON_ACTIVE_FG,
+                     COLOR_TRAYBUTTON_ACTIVE_BG1, COLOR_TRAYBUTTON_ACTIVE_BG2,
+                     COLOR_TRAYBUTTON_ACTIVE_UP, COLOR_TRAYBUTTON_ACTIVE_DOWN);
+         break;
+      case TOK_BACKGROUND:
+         ParseGradient(np->value, COLOR_TRAYBUTTON_BG1, COLOR_TRAYBUTTON_BG2);
+         break;
+      case TOK_FOREGROUND:
+         SetColor(COLOR_TRAYBUTTON_FG, np->value);
+         break;
+      case TOK_OUTLINE:
+         ParseGradient(np->value, COLOR_TRAYBUTTON_DOWN, COLOR_TRAYBUTTON_UP);
+         break;
+      default:
+         InvalidTag(np, TOK_TRAYBUTTONSTYLE);
+         break;
+      }
+   }
 }
 
 /** Parse a swallow tray component. */
@@ -1448,7 +1553,9 @@ void ParsePagerStyle(const TokenNode *tp)
          SetColor(COLOR_PAGER_BG, np->value);
          break;
       case TOK_ACTIVE:
-         ParseActivePagerStyle(np);
+         ParseActive(np, COLOR_PAGER_ACTIVE_FG,
+                     COLOR_PAGER_ACTIVE_BG, COLOR_PAGER_ACTIVE_BG,
+                     COLOR_COUNT, COLOR_COUNT);
          break;
       case TOK_FONT:
          SetFont(FONT_PAGER, np->value);
@@ -1464,25 +1571,6 @@ void ParsePagerStyle(const TokenNode *tp)
 
 }
 
-/** Parse active pager style. */
-void ParseActivePagerStyle(const TokenNode *tp)
-{
-   const TokenNode *np;
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FOREGROUND:
-         SetColor(COLOR_PAGER_ACTIVE_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         SetColor(COLOR_PAGER_ACTIVE_BG, np->value);
-         break;
-      default:
-         InvalidTag(np, TOK_ACTIVE);
-         break;
-      }
-   }
-}
-
 /** Parse clock style. */
 void ParseClockStyle(const TokenNode *tp)
 {
@@ -1569,7 +1657,7 @@ void ParseMenuStyle(const TokenNode *tp)
 {
    const TokenNode *np;
 
-   settings.menuDecorations = ParseDecorations(tp);
+   ParseDecorations(tp, &settings.menuDecorations);
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_FONT:
@@ -1582,7 +1670,9 @@ void ParseMenuStyle(const TokenNode *tp)
          SetColor(COLOR_MENU_BG, np->value);
          break;
       case TOK_ACTIVE:
-         ParseActiveMenuStyle(np);
+         ParseActive(np, COLOR_MENU_ACTIVE_FG,
+                     COLOR_MENU_ACTIVE_BG1, COLOR_MENU_ACTIVE_BG2,
+                     COLOR_MENU_ACTIVE_UP, COLOR_MENU_ACTIVE_DOWN);
          break;
       case TOK_OUTLINE:
          ParseGradient(np->value, COLOR_MENU_DOWN, COLOR_MENU_UP);
@@ -1598,25 +1688,6 @@ void ParseMenuStyle(const TokenNode *tp)
 
 }
 
-/** Parse active menu style. */
-void ParseActiveMenuStyle(const TokenNode *tp)
-{
-   const TokenNode *np;
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FOREGROUND:
-         SetColor(COLOR_MENU_ACTIVE_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_MENU_ACTIVE_BG1, COLOR_MENU_ACTIVE_BG2);
-         break;
-      default:
-         InvalidTag(np, TOK_ACTIVE);
-         break;
-      }
-   }
-}
-
 /** Parse an option group. */
 void ParseGroup(const TokenNode *tp)
 {
@@ -1683,19 +1754,18 @@ void ParseGroupOption(const TokenNode *tp, struct GroupType *group,
 }
 
 /** Parse decorations type. */
-DecorationsType ParseDecorations(const TokenNode *tp)
+void ParseDecorations(const TokenNode *tp, DecorationsType *deco)
 {
    const char *str = FindAttribute(tp->attributes, "decorations");
    if(str) {
       if(!strcmp(str, "motif")) {
-         return DECO_MOTIF;
+         *deco = DECO_MOTIF;
       } else if(!strcmp(str, "flat")) {
-         return DECO_FLAT;
+         *deco = DECO_FLAT;
       } else {
          ParseError(tp, _("invalid decorations: %s"), str);
       }
    }
-   return DECO_FLAT;
 }
 
 /** Parse a color which may be a gradient. */
@@ -1800,18 +1870,27 @@ char *ReadFile(FILE *fd)
 
    for(;;) {
       const size_t count = fread(&buffer[len], 1, max - len, fd);
+      if(count == 0) {
+         if(feof(fd)) {
+            break;
+         } else if(errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR) {
+            continue;
+         } else {
+            Warning(_("could not read file: %s"), strerror(errno));
+            break;
+         }
+      }
       len += count;
-      if(len < max) {
-         break;
-      }
-      max += BLOCK_SIZE;
-      if(JUNLIKELY(max < 0)) {
-         /* File is too big. */
-         break;
-      }
-      buffer = Reallocate(buffer, max + 1);
-      if(JUNLIKELY(buffer == NULL)) {
-         FatalError(_("out of memory"));
+      if(len == max) {
+         max += BLOCK_SIZE;
+         if(JUNLIKELY(max < 0)) {
+            /* File is too big. */
+            break;
+         }
+         buffer = Reallocate(buffer, max + 1);
+         if(JUNLIKELY(buffer == NULL)) {
+            FatalError(_("out of memory"));
+         }
       }
    }
    buffer[len] = 0;
